@@ -7,6 +7,7 @@
                      '(("fontenc" "T1") ("inputenc" "utf8") ("natbib" "round") ("numprint" "autolanguage") ("helvet" "scaled=0.9") ("mathpazo" "sc") ("enumitem" "shortlabels") ("Sweave" "noae" "inconsolata")))
    (add-to-list 'LaTeX-verbatim-environments-local "Verbatim")
    (add-to-list 'LaTeX-verbatim-environments-local "lstlisting")
+   (add-to-list 'LaTeX-verbatim-macros-with-braces-local "href")
    (add-to-list 'LaTeX-verbatim-macros-with-braces-local "hyperref")
    (add-to-list 'LaTeX-verbatim-macros-with-braces-local "hyperimage")
    (add-to-list 'LaTeX-verbatim-macros-with-braces-local "hyperbaseurl")
@@ -62,13 +63,13 @@
     "eq:pdf-YP")
    (LaTeX-add-bibliographies
     "actuar")
+   (LaTeX-add-amsthm-newtheorems
+    "example"
+    "rem")
    (LaTeX-add-xcolor-definecolors
     "link"
     "url"
     "citation"
-    "codebg")
-   (LaTeX-add-amsthm-newtheorems
-    "example"
-    "rem"))
+    "codebg"))
  :latex)
 
